# MoneyCycles.R
#
# Copyright (C) 2009-2014 Andrew Clausen <andrew.clausen@ed.ac.uk> and
# Carlo Strub <cs@carlostrub.ch>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted without any restrictions.

# NOTATION throughout:
# _ suffixes are like ' in normal math language.  For example, c_ is like c'
# (marginal cost) and a_ is like a' (savings, i.e. future asset holdings).

# tolerance for finding roots
tol <- 1e-8

crra.utility <- function()
{
	# utility function, and related functions (marginal utility, inverse,
	# etc.)
	u <- function(x) delta * x^alpha
	u_ <- function(x) ifelse(x < 0, Inf, delta * alpha * x^(alpha-1))
	u_.inverse <- function(u) (u/alpha/delta)^(1/(alpha - 1))
	list(u=u, u_=u_, u_.inverse=u_.inverse)
}

cara.utility <- function()
{
	# utility function, and related functions (marginal utility, inverse,
	# etc.)
	u <- function(x) - delta * exp(- x*alpha)
	u_ <- function(x) delta * alpha * exp(- x*alpha)
	u_.inverse <- function(u) - log(u/alpha/delta) / alpha
	list(u=u, u_=u_, u_.inverse=u_.inverse)
}

quadratic.utility <- function()
{
	u <- function(x) delta * (x - x^2 * alpha)
	u_ <- function(x) ifelse(x < 0, Inf, delta * (1 - 2 *alpha * x))
	u_.inverse <- function(u) -(u-1)/(2*alpha*delta)
	list(u=u, u_=u_, u_.inverse=u_.inverse)
}

cobb.douglas.cost <- function()
{
	# cost function, and related functions
	cost.variable <- function(q) q^gamma
	cost <- function(q) ifelse(q == 0, 0, cost.fixed + cost.variable(q))
	cost_ <- function(q) gamma * q^(gamma - 1)
	cost_.inverse <- function(cost) (cost/gamma)^(1/(gamma-1))
	list(cost.variable=cost.variable, cost=cost, cost_=cost_,
	     cost_.inverse=cost_.inverse)
}

# calculates the (efficient) equilibrium of the static convexified economy
convexified.eq <- function()
{
	q <- uniroot(function(q) cost_(q) - cost(q)/q,
			c(0.00001, 100), tol=tol)$root
	x <- uniroot(function(x) u_(x) - cost_(q), c(0, 100), tol=tol)$root
	list(q=q, x=x)
}

# marginal utility of the consumption made t periods, not including
# discounting.  (from the Euler equation)
u_.equilibrium <- function(startx,t) (beta/(1+pi))^(-t) * u_(startx)

# consumption at period t of the cycle
x.equilibrium <- function(startx,t) u_.inverse(u_.equilibrium(startx,t))

# (shadow) marginal cost from first-order conditions.
# Here, "shadow" means the marginal cost conditional on producing.  On vacation
# days, there is no marginal cost, but if you were forced to work on that day,
# what would the marginal cost of your optimal quantity be?
cost_.shadow <- function(startx,t) u_.equilibrium(startx,t)

# (shadow) production quantity
q.shadow <- function(startx,t) cost_.inverse(cost_.shadow(startx,t))

# inverse of q.shadow for t=0. (given startq, compute startx)
q.shadow.inv <- function(startq) u_.inverse(cost_(startq))

# calculate bounds for maximum cycle length.  (See the appendix for an
# explanation.)
calc.bounds <- function(T)
{
	min.startx <- T

	# from proof of Theorem 3, part (i)
	psi <- function(x) u_(x) - cost_(x - T)
	max.startx <- uniroot(psi, c(min.startx, 100), tol=tol)$root

	# since u' = c', can relate bounds on x to bounds on q.
	min.startq <- q.shadow(max.startx, 0)
	max.startq <- q.shadow(min.startx, 0)

	# from Theorem 3, part (ii)
	max.n <- ceiling(log(u_(max.startx) / u_(T)) / log(beta / (1+pi)))

	list(min.startx=min.startx, max.startx=max.startx,
	     min.startq=min.startq, max.startq=max.startq,
	     max.n=max.n, T=T)
}

# Calculates the greedy solution to the household's problem
# A greedy plan is one in which workers take vacations whenever they
# can afford their optimal amount of consumption that period.
greedy.cycle <- function(bounds, n, startx)
{
	x <- numeric(n)
	z_ <- numeric(n)
	q <- numeric(n)

	z <- 0
	for (t in 1:n)
	{
		z <- z + bounds$T
		x[[t]] <- x.equilibrium(startx, t-1)
		if (z < x[[t]])
		{
			q[[t]] <- q.shadow(startx, t-1)
			z <- z + q[[t]]
		}
		if (z < x[[t]])
			return(list(value=-Inf))
		z <- (z - x[[t]]) / (1 + pi)
		z_[[t]] <- z
	}

	z <- c(0, z_[1:(n-1)])
	t <- 0:(n-1)
	value <- 1 / (1 - beta^n) * sum(beta^t * (u(x) - cost(q)))
	list(n=n, x=x, q=q, z=z, z_=z_, value=value)
}

# calculates how much money the agent has left over at the end of the cycle
# (should be zero)
budget <- function(eq)
{
	t <- 0:(eq$n - 1)
	sum((eq$q + eq$T - eq$x) * (1 + pi)^t)
}

# calculates the optimal cycle of length n, given the bounds
optimal.cycle.given.n <- function(bounds, n)
{
	startx <- optimize(f = function(startx) greedy.cycle(bounds, n, startx)$value,
			   lower = bounds$min.startx,
			   upper = bounds$max.startx,
			   maximum = TRUE, tol=tol)$maximum
	cycle <- greedy.cycle(bounds, n, startx)
	stopifnot(abs(budget(cycle)) < 0.001)
	cycle
}

# finds plan and startx with highest value.
optimal.cycle <- function(T, n.limit=Inf)
{
	bounds <- calc.bounds(T)
	if (bounds$min.startx >= bounds$max.startx)
		return(list(value=-Inf))
	cycles <- lapply(1:min(n.limit, bounds$max.n),
			 function(n) optimal.cycle.given.n(bounds, n))
	best.n <- which.max(sapply(cycles, function(cycle) cycle$value))
	cycles[[best.n]]
}

# excess demand for a given (potentially non-equilibrium) real value of
# transfers, T
excess.demand <- function(T, n.limit=30)
{
	cycle <- optimal.cycle(T, n.limit=n.limit)
	cycle$excessdemand <- (sum(cycle$x) - sum(cycle$q)) / cycle$n
	cycle
}

# calculate equilibrium by adjusting prices such that markets clear
calc.eq <- function()
{
	eq.convex <- convexified.eq()
	ed <- function(T) excess.demand(T)$excessdemand
	T <- uniroot(ed, c(0, eq.convex$q), tol=tol)$root
	eq <- optimal.cycle(T, n.limit=30)
	eq$T <- T

	# measure 1/n agents at each stage of the plan; should equal eq$T / pi
	eq$Z <- sum(eq$z_) / eq$n

	eq
}

# write the data out ready for LaTeX to use
format.pstricks <- function(x, y, example.name, name)
{
	points <- cbind(x, y)
	points.pairs <- apply(
		points,
		1,
		function(x) paste("{", x[1], ",", x[2], "}"))

	paste("\\savedata{\\", example.name, name, "}[{",
	      paste(points.pairs, collapse=","), "}]\n",
	      sep="")
}
write.latex.data <- function(eq, example.name)
{
	# create psTricks vectors
	transform.t <- function(t) t * 85/(length(times)-1) + 5
	transform.x <- function(x) x * 85/range(eq$q)[2] + 5
	transform.z <- function(x) x * 85/range(eq$z)[2] + 5

	times <- 1:(2*eq$n + 1) -1
	format.timeseries <- function(x, name)
		format.pstricks(transform.t(times),
			        transform.x(x[times %% eq$n + 1]),
				example.name, name)

	cat(format.timeseries(eq$x, "cyclex"), file="graph", append=TRUE)
	cat(format.timeseries(eq$q, "cycleq"), file="graph", append=TRUE)
	cat(format.timeseries(eq$z, "cyclez"), file="graph", append=TRUE)

	cat(format.pstricks(transform.z(sort(eq$z_)), rep(20, eq$n),
		example.name, "moneyhist"), file="graph", append=TRUE)

	cat(paste("\\newcommand{\\", example.name, "dx}{",
	    85/(length(times) - 1), "}\n", sep=""), file="graph", append=TRUE)

	eqconv <- convexified.eq()
	cat(paste("\\newcommand{\\", example.name, "convx}{",
		  transform.x(eqconv$x), "}\n", sep=""),
		  file="graph", append=TRUE)
	cat(paste("\\newcommand{\\", example.name, "convq}{",
		  transform.x(eqconv$q), "}\n", sep=""),
		  file="graph", append=TRUE)
}

# Plot money cycles
plot.eq <- function(eq)
{
	plot(rep(eq$x, 2), type="b", col="blue", ylim=c(0, max(eq$q)),
	     xlab="t", ylab="")
	lines(rep(eq$z_, 2), type="b", col="green")
	lines(rep(eq$q, 2), type="b", col="red")
	legend("topright", c("x", "z", "q"), lty=c(1, 1, 1),
	       col=c("blue", "green", "red"))
}

# Plot excess demand
ed.plot <- function(eqt)
{
	l <- 100
	delta <- eqt*.2
	T.sequence <- c(seq(eqt-delta,eqt,length.out=l/2),
				seq(eqt+delta,eqt,length.out=l/2))
	out <- lapply(T.sequence,excess.demand)
	exdemand <- sapply(seq_len(l),function(index) out[[index]]$excessdemand)
	cyclen <- sapply(seq_len(l),function(index) out[[index]]$n)

	list(seq=T.sequence,ed=exdemand,cycle=cyclen)
}

# exogenous parameters for simple example
simple.example <- c(
	name="simple",
	crra.utility(), 
	cobb.douglas.cost(),
	alpha=0.3,
	beta=0.98,
	delta=1,
	gamma=1.1,
	cost.fixed=.1,
	pi=0.01
)

# exogenous parameters for quadratic example
stimulus.example <- c(
	name="stimulus",
	crra.utility(), 
	cobb.douglas.cost(),
	alpha=0.3,
	beta=0.98,
	delta=1,
	gamma=1.3,
	cost.fixed=.15,
	pi=0.3
)

# exogenous parameters for complex example
complex.example <- c(
	name="complex",
	crra.utility(), 
	cobb.douglas.cost(),
	alpha=0.3,
	beta=0.98,
	delta=1,
	gamma=1.5,
	cost.fixed=.1,
	pi=0.01
)

# exogenous parameters for complex example with pi <- 0.01446 (for excess demand
# plot)
excessdemand.example <- c(
	name="excessdemand",
	crra.utility(), 
	cobb.douglas.cost(),
	alpha=0.3,
	beta=0.98,
	delta=1,
	gamma=1.5,
	cost.fixed=.1,
	pi=0.01445
)

cara.example <- c(
	name="cara",
	cara.utility(), 
	cobb.douglas.cost(),
	alpha=0.5,
	beta=0.98,
	delta=4,
	gamma=1.5,
	cost.fixed=.5,
	pi=0.2
)

### Write LaTeX file for use inside the paper
cat("", file="graph")
for (example in list(simple.example, stimulus.example, cara.example, complex.example))
{
	attach(example,
	       warn.conflicts=FALSE)	# We redefine "pi", to R's disgust
	cat(paste("Calculating", name, "example...\n"))
	eq <- calc.eq()
	plot.eq(eq)
	write.latex.data(eq, name)
	detach()
}


## Plot excess demand and write it to LaTeX file
attach(excessdemand.example, warn.conflicts=FALSE)
cat(paste("Calculating", name, "example...\n"))
eq <- calc.eq()
ed.result <- ed.plot(eq$T)
cat(format.pstricks(ed.result$seq[1:50]*100000-135,
	ed.result$ed[1:50]*40000 + 35,"ed", "one"),
	file="graph", append=TRUE)
cat(format.pstricks(ed.result$seq[51:99]*100000-135,
	ed.result$ed[51:99]*40000 + 35,"ed", "two"),
	file="graph", append=TRUE)
cat(paste("\\newcommand{\\edeqT}{", eq$T*100000-135, "}\n", sep=""),
	file="graph", append=TRUE)
cat(paste("\\newcommand{\\edeqTone}{", ed.result$ed[[50]]*40000+35, "}\n", sep=""),
	file="graph", append=TRUE)
cat(paste("\\newcommand{\\edeqTtwo}{", ed.result$ed[[99]]*40000+35, "}\n", sep=""),
	file="graph", append=TRUE)
detach()

